#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#ifdef BSD
#include <sys/elf32.h>
#include <sys/elf64.h>
#endif
#ifdef LINUX
#include <elf.h>
#else
#error READ THE README FILE!
#endif
#include <fcntl.h>
#include <ncurses.h>
#include "elfe.h"

WINDOW *mainw,*optw,*ow,*logo; 
Elf32_Ehdr *Elfhdr32;
Elf64_Ehdr *Elfhdr64;

void *filec = 0;

void Sec_opt(void);
void Sym_opt(void);

void Quit(void)
{
    wclear(mainw);
    endwin();
    exit(0);
}
    
struct mainopt { 
    int l;
    char *desc;
    void (*func)(void);
}opts[] = {
    {0,"Section",Sec_opt},
    {1,"Symbol",Sym_opt},
    {2,"Quit.",Quit},
    {NULL,NULL,NULL}
};

void helpmenu(void){ 

    printf("Usage: elfe <file>\n");
    printf("File -> ELF file to edit\n");
    printf("or elfe --help which will give you this menu :)\n");
    return;
}

int is32bit = 0;
long fsize;
int fd;
char *file = 0;

int main(argc,argv)
    int argc;
    char **argv;
{ 

    struct mainopt *cp = &opts[0];    
    int i,j;
    
    
    for(i = 1;argv[i];i++) { 
	if(argv[i][0] != '-') { 
	    if(!file) { 
		file = strdup(argv[i]);
		continue;
	    }else { 
		printf("Invalid argument %s\n",argv[i]);
		return ERROR;
	    }
	
	}
	
	if(!strcmp(argv[i],"--help")) { 
	    helpmenu();
	    return ERROR;
	}
    }
    
    if(!file) { 
	printf("Give me a file name to edit please.\n");
	return ERROR;
    } 

    if(( fd = open(file,O_RDWR)) < 0) { 
	perror(file);
	return ERROR;
    }
    
    fsize = lseek(fd,0,SEEK_END);
#ifdef MEM_PROTECT
    if(fsize > MEM_PROTECT) { 
	printf("Size 0x%lx is too big for me to handle\n",fsize);
	return ERROR;
    }
#endif

    filec = (void*)malloc(fsize);
    
    if(!filec) { 
	perror("malloc");
	return ERROR;
    }

    lseek(fd,0,SEEK_SET);
    if(read(fd,filec,fsize) != fsize) {
	printf("Unable to read the full file.\n");
	return ERROR;
    }
    
    if((is32bit = getelftype(fd)) < 0) { 
	printf("Unknown type of elf only supported 32bit and 64bit\n");
	return ERROR;
    }
    
    if(is32bit)
	Elfhdr32 = (Elf32_Ehdr *)filec;
    else
	Elfhdr64 = (Elf64_Ehdr *)filec;
	
    init_win();
    
    /* printing out logo window */
    wmove(logo,0,1);
    waddstr(logo,LOGO);
    wrefresh(logo); 
    wmove(logo,1,1);
    wprintw(logo,"%s:	file format Elf%d size 0x%x",file,is32bit?32:64,fsize);
    wrefresh(logo); 
    
#define MARKOPT\
	wclear(optw);\
        for(i = 0;opts[i].func;i++){\
		wmove(optw,opts[i].l,1);\
		waddstr(optw,opts[i].desc);\
	}\
	wattrset(optw,COLOR_PAIR(HIGHLIGHT));\
	wmove(optw,cp->l,1);\
	waddstr(optw,cp->desc);\
	wattroff(optw,COLOR_PAIR(HIGHLIGHT));\
	wrefresh(optw);


    MARKOPT
    j = 0;
    while(1) {
	switch(wgetch(optw)) { 
	    case KEY_DOWN:
		if(j == 2) break;
		j++;
		cp = &opts[j];
		MARKOPT
		break;
	    case KEY_UP:
		if(j == 0) break;
		j--;
		cp = &opts[j];
		MARKOPT
		break;
	    case KEY_RIGHT:
		opts[j].func();
		MARKOPT
		break;
	    case 'q':
		goto end;
	    default:
		break;
	};
    }	
end:
    wclear(mainw);
    endwin();
    return 1;    
} 

int getelftype(int fd)
{ 
    Elf32_Ehdr *Ehdr;

    
    Ehdr = (Elf32_Ehdr *)filec;
    if(Ehdr->e_ident[EI_CLASS] == ELFCLASS32)
	return 1;
    else if(Ehdr->e_ident[EI_CLASS] == ELFCLASS64)
	return 0;
    else 
	return -1;
}        

int save()
{
    lseek(fd,0x0,SEEK_SET);
    write(fd,filec,fsize);
    return 1;
}
        